;;; jde-complete-tooltip.el --- 

;; $Id: jde-complete-tooltip.el,v 1.6 2002/12/09 13:56:15 burton Exp $

;; Copyright (C) 2000-2003 Free Software Foundation, Inc.
;; Copyright (C) 2000-2003 Kevin A. Burton (burton@openprivacy.org)

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords: 
;; Version: 1.0.0

;; This file is [not yet] part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2 of the License, or any later version.
;;
;; This program is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along with
;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
;; Place - Suite 330, Boston, MA 02111-1307, USA.

;;; Commentary:

;; NOTE: If you enjoy this software, please consider a donation to the EFF
;; (http://www.eff.org)

;; write my own impl of jde-complete-get-classinfo so that I can:
;;
;; - determine public and private methods
;;
;; - handle my own sorting
;;
;; - boldify methods in the implementing class
;;
;; - underline static methods
;;
;; - italicize abstract methods

;;; Code:

(defface jcapt-function-name-face '((t (:inherit 'tooltip :foreground "Navy" :bold t)))
  "Face used for function names")

(defface jcapt-keyword-face '((t (:inherit 'tooltip :foreground "DarkCyan" :bold t)))
  "Face used for keyword names")

(defface jcapt-type-face '((t (:inherit 'tooltip :foreground "DarkGreen" :bold t)))
  "Face used for type names")

(defface jcapt-constant-face '((t (:inherit 'tooltip :foreground "aquamarine4" :bold t)))
  "Face used for constant names")

(defface jcapt-variable-name-face '((t (:inherit 'tooltip :foreground "DarkGoldenrod" :bold t)))
  "Face used for variable names")

(defun jde-complete-at-point-tooltip()
  "Test a key binding for event"
  (interactive)

  (let((jde-complete-insert-method-signature nil)
       (completion-list (jde-complete-tooltip-get-completion-list)))

    (when (equal 0 (length completion-list))
      (error "No completions found"))
    
    (tooltip-at-point (format "%s" (jde-complete-tooltip-get-completion-content completion-list)))

  ;;if we only have one alternative... insert it.

    (when (= 1 (length completion-list))

      (when (re-search-backward "[.]*" nil t)
        
        (delete-region (match-beginning 0) (match-end 0))
           
        (insert "." (cdr (nth 0 completion-list)))))))

(defun jde-complete-tooltip-get-completion-content(completion-list)
  "Get the completion list as a string."  
  
  (save-excursion
    (set-buffer (get-buffer-create "*jde-complete-tooltip*"))
    (erase-buffer)

    ;;sort methods
;;     (setq completion-list (sort  completion-list '(lambda(one two)
;;                                                     (not (string-match "(" (car one))))))
                                                     
    (dolist(entry completion-list)

      (insert (jde-complete-tooltip--fontify (car entry)))
      (insert "\n"))

    (buffer-substring (point-min) (point-max))))

(defun jde-complete-tooltip--fontify(entry)
  "Fontify the given entry."

  (when entry

    (let((case-fold-search nil)
         (index 0))
      (while (string-match "void\\|byte\\|int\\|double\\|long\\|char\\|boolean" entry index)
        (add-text-properties (match-beginning 0)
                             (match-end 0)
                             '(face font-lock-type-face) entry)

        (setq index (match-end 0))))
    
    (let((case-fold-search t))
      (when (string-match "^\\([a-zA-0-9]+\\)(" entry)
        
        (add-text-properties (match-beginning 1)
                             (match-end 1)
                             '(face font-lock-function-name-face) entry)))

    (let((case-fold-search t))
      (when (string-match "^\\([a-zA-0-9]+\\) :" entry)
        (add-text-properties (match-beginning 1)
                             (match-end 1)
                             '(face font-lock-variable-name-face) entry)))

    (when (string-match "throws" entry)
      (add-text-properties (match-beginning 0)
                           (match-end 0)
                           '(face font-lock-keyword-face) entry))

    (let((case-fold-search nil)
         (index 0))
      (while (string-match "[( ]\\(\\([a-z]+\\.\\)+\\([A-Z]*[a-z0-9]+\\)*\\)" entry index)

        (add-text-properties (match-beginning 1)
                             (match-end 1)
                             '(face font-lock-constant-face) entry)

        (setq index (match-end 0))))

    )

  entry)

(defun jde-complete-tooltip-get-completion-list()
  "Completes the method or field name at point.  This command displays a popup
menu listing the potential completions for the name at point. Selecting a
completion causes the command to use the completion to complete the name at
point. See `jde-complete-at-point' for a version of this command that lets you
cycle throught the potential completions at point."
  (interactive)

  (let* ((pair (jde-complete-java-variable-at-point))
         jde-complete-attempted-to-import
         completion-list
         access)

    ;;resetting jde-complete-cu
    (setq jde-complete-current-list nil)
    (if pair
        (progn
          (if (string= (car pair) "" )
              (progn
                (setcar pair "this")))
          (if (string= (car pair) "this")
              (setq access jde-complete-private)
            (if (string= (car pair) "super")
                (setq access jde-complete-protected)))
          (if access 
              (setq completion-list (jde-complete-find-completion-for-pair pair nil
                                                                           access))
            (setq completion-list (jde-complete-find-completion-for-pair pair)))
          ;;if the completion list is nil check if the method is in the current
          ;;class(this)
          (if (null completion-list)
              (setq completion-list (jde-complete-find-completion-for-pair
                                     (list (concat "this." (car pair)) "")
                                     nil jde-complete-private)))
          ;;if completions is still null check if the method is in the super class
          (if (null completion-list)
              (setq completion-list (jde-complete-find-completion-for-pair
                                     (list (concat "super." (car pair)) "")
                                     nil jde-complete-protected)))
          
          completion-list))))

(provide 'jde-complete-tooltip)

;;; jde-complete-tooltip.el ends here
