;;; jhideshow.el --- ADD BASIC DESCRIPTION

;; $Id: jhideshow.el,v 1.3 2003/01/10 05:57:59 burton Exp $

;; Copyright (C) 2000-2003 Free Software Foundation, Inc.
;; Copyright (C) 2000-2003 Kevin A. Burton (burton@openprivacy.org)

;; Author: Kevin A. Burton (burton@openprivacy.org)
;; Maintainer: Kevin A. Burton (burton@openprivacy.org)
;; Location: http://relativity.yi.org
;; Keywords: 
;; Version: 1.0.0

;; This file is [not yet] part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify it under
;; the terms of the GNU General Public License as published by the Free Software
;; Foundation; either version 2 of the License, or any later version.
;;
;; This program is distributed in the hope that it will be useful, but WITHOUT
;; ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
;; FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
;; details.
;;
;; You should have received a copy of the GNU General Public License along with
;; this program; if not, write to the Free Software Foundation, Inc., 59 Temple
;; Place - Suite 330, Boston, MA 02111-1307, USA.

;;; Commentary:
;;
;; If you want jhideshow to be turned on by default you just need to add the
;; following in your .emacs file.
;;
;; (add-hook 'semantic-after-toplevel-bovinate-hook 'turn-on-jhideshow)

;;; TODO:
;;
;; - Documentation
;;
;; - What about abstract classes?  
;;
;; - Need the ability to make SURE that semantic has at least run once and that
;; it is not dirty...


;;; History: 
;;
;; - (DONE)Methods that only take up one line screw us up... 100% CPU utilization.
;;
;;    -  public StylesheetStack() { }
;;
;; - (DONE) Don't do anything with interfaces (is there any way to find this out?)
;;

;;; Code:

(require 'semantic)
(require 'jde)

(defvar jhideshow-mode nil "True if `jhidehow-mode' is on.")
(make-variable-buffer-local 'jhideshow-mode)

(defun turn-on-jhideshow()
  "Turn on jhideshow if "
  (interactive)

  ;;turn on jhideshow only if we are in jde-mode *and* jhideshow-mode is off..
  (if (and (equal major-mode 'jde-mode)
           (equal jhideshow-mode nil))
      (jhideshow-mode-on)))

(defun jhideshow-mode-off()
  "Turn off jhideshow mode."
  (interactive)
  
  (hs-show-all)

  (hs-minor-mode)

  (setq jhideshow-mode nil))

(defun jhideshow-mode-on()
  "Turn on jhideshow mode."
  (interactive)

  (hs-minor-mode 1)

  ;;(hs-hide-all)

  (setq jhideshow-mode t))

(defun jhideshow-interface-p()
  "Return true if we are in a java interface."

  (save-excursion
    (goto-char (point-min))

    ;;This is a HACK of a way to check for interfaces...
    (re-search-forward "^public interface" nil t)))

(defun jhideshow-token-function-p(token)
  "Return true if this token is a function."
  
  (equal (semantic-token-token token) 'function))

(defun jhideshow-goto-function-body-start() 

  (goto-char (point-at-bol))
  
  (search-forward "{" nil t)

  (goto-char (match-end 0)))
  
(defun jhideshow-next-token ()
  "Navigate to the next Semantic token.
Return the token or nil if at end of buffer."

  (let ((pos   (point))
        (token (semantic-current-nonterminal))
        where)
    (if (and token
             (not (senator-skip-p token))
             (senator-step-at-start-end-p token)
             (or (= pos (semantic-token-start token))
                 (senator-middle-of-token-p pos token)))
        nil
      (if (setq token (senator-step-at-parent token))
          nil
        (setq token (semantic-find-nonterminal-by-overlay-next pos))
        (while (and token (senator-skip-p token))
          (setq token (semantic-find-nonterminal-by-overlay-next
                       (semantic-token-start token))))))
    (if (not token)
        (progn
          (goto-char (point-max))
          (working-message "End of buffer"))
      (cond ((and (senator-step-at-start-end-p token)
                  (or (= pos (semantic-token-start token))
                      (senator-middle-of-token-p pos token)))
             (setq where "end")
             (goto-char (semantic-token-end token)))
            (t
             (setq where "start")
             (goto-char (semantic-token-start token))))

      )
    token))

(defun jhideshow-hide-current-token()
  "Hide the current semantic token.  This provides the ability to hide on a
token level and not on a block level which is the default in hideshow."
  (interactive)
  
  (goto-char (semantic-token-start (semantic-current-nonterminal)))

  (jhideshow-goto-function-body-start)

  (hs-hide-block))

(defun jhideshow-hide-all()
  "Hide all tokens."
  (interactive)

  (semantic-bovinate-toplevel)
  
  (if (not (jhideshow-interface-p))
      
      (save-excursion

        (goto-char (point-min))
        
        (while (not (equal (point) (point-max)))
          
          (jhideshow-next-token)
      
          (let(token)

            (setq token (semantic-current-nonterminal))

            (goto-char (point-at-eol))
        
            ;;(jhideshow-goto-function-body-start)
        
            (if (jhideshow-token-function-p token)
                (progn

                  (jhideshow-goto-function-body-start)
              
                  (hs-hide-block)

                  (goto-char (point-at-eol))
                  
                  (forward-line 1))))))))

(defun jhideshow-show-all()
  "Show all tokens."
  (interactive)

  (hs-show-all))

(provide 'jhideshow)

;;; jhideshow.el ends here
